SystemOrganization addCategory: #TusLibros!

TestCase subclass: #CreditCardTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithDataOKShouldReturnTheCard

	| creditCard expectedNumber expectedOwner expectedExpirationDate |
	
	expectedNumber := '1234'.
	expectedOwner := 'Homero Simpson'.
	expectedExpirationDate := Date tomorrow.
	
	creditCard := CreditCard newWithNumber: expectedNumber andOwner: expectedOwner andExpirationDate: expectedExpirationDate.
	
	self assert: creditCard number equals: expectedNumber.
	self assert: creditCard owner equals: expectedOwner.
	self assert: creditCard expirationDate equals: expectedExpirationDate.! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithEmptyNumberShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card number cannot be empty.'] ! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:01'!
testCreatingACreditCardWithEmptyOwnerNameShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be empty.'] ! !

!CreditCardTest methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:05'!
testCreatingACreditCardWithOwnerNameLengthGreaterThan30ShouldRaiseAnError

	 self should: [CreditCard newWithNumber: '1234' andOwner: '123456789_123456789_123456789_1' andExpirationDate: Date tomorrow] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = 'Credit card owner cannot be greater than 30 characters long.'] ! !

TestCase subclass: #MerchantProcessorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessorTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:22'!
creditCardExpiredResult

	^ CreditCardTransactionResult newWithCode: '1' andMessage: 'EXPIRED_CARD'.! !

!MerchantProcessorTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:22'!
okResult

	^ CreditCardTransactionResult ok.! !

!MerchantProcessorTest methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:19'!
testDebit0AmountShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := MerchantProcessorStub new.
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor debit: 0 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self wrongAmountResult.! !

!MerchantProcessorTest methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:18'!
testDebitNegativeAmountShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := MerchantProcessorStub new.
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor debit: -100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self wrongAmountResult.! !

!MerchantProcessorTest methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:19'!
testDebitWithDataOKShouldReturnOK

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := MerchantProcessorStub new.
	creditCard := CreditCard newWithNumber: '1234' andOwner: 'Homero Simpson' andExpirationDate: Date tomorrow.
	
	responseFromMerchantProcessor := merchantProcessor debit: 100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self okResult.! !

!MerchantProcessorTest methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:20'!
testDebitWithExpiredCardShouldReturnError

	| merchantProcessor creditCard responseFromMerchantProcessor|
	
	merchantProcessor := MerchantProcessorStub new.
	creditCard := CreditCard newWithNumber: '5678' andOwner: 'Marge Simpson' andExpirationDate: Date yesterday.
	
	responseFromMerchantProcessor := merchantProcessor debit: 100.50 fromCard: creditCard.
	
	self assert: responseFromMerchantProcessor equals: self creditCardExpiredResult.! !

!MerchantProcessorTest methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:22'!
wrongAmountResult

	^ CreditCardTransactionResult newWithCode: '1' andMessage: 'AMOUNT_MUST_BE_GREATER_THAN_ZERO'.! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 21:46'!
carrito
	
		^ Cart newCartWithCatalog: self catalogo! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:00'!
carritoConUnLibro
     | book1 catalog |
	book1 := '3514564564674'.
	catalog := self catalogo.
	
	^ (Cart newCartWithCatalog: catalog) add: book1! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:07'!
catalogo
	| book1 book2 catalog book3 |
	
	book1 := '3514564564674'.
	book2 := '6464564564564'.
	book3:= '35145645646748'.	
	
	catalog := Dictionary new.
	catalog at: book1 put: 2.
	catalog at: book2 put: 25.
	catalog at: book3 put: 27.
	
	^ catalog! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/27/2013 20:44'!
test01AllCartsAreCreatedEmpty
	"Todo carrito se crea vaco"

	self assert: self carrito isEmpty.! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:01'!
test02AfterAddingTheFirstBookCartShouldNotBeEmpty
	"Luego de agregarle libros, el carrito no est vaco"

	self deny: (self carrito add: '6464564564564' withQuantity: 1) isEmpty.! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:05'!
test03AfterAddingMoreBooksCartShouldStillNotBeEmpty
	"Luego de agregarle libros, el carrito no est vaco"

	| aBook aCartWithBooks|
	
	aBook:= '3514564564674'.
	
	aCartWithBooks:=  self carrito add: aBook withQuantity: 1.
	
	self deny: (aCartWithBooks add: aBook withQuantity: 1) isEmpty.
	! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
test03BooksAddedShouldAppearWithTheirQuantities
	
	| aBook aCartWithBooks|
	
	aBook:= '35145645646748'.
	
	aCartWithBooks:=  self carrito add: aBook withQuantity: 1.
	aCartWithBooks add: aBook withQuantity: 1.
		
	self assert: (aCartWithBooks hasItem: aBook  withQuantity: 2)! !

!TusLibrosTest methodsFor: 'cart tests' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
test03CartShouldOnlyHaveTheBooksThatWereAdded
	
	| book1 book2 aCartWithBooks|
	
	book1:= '35145645646748'.
	book2:= '6464564564564'.
	
	aCartWithBooks:=  self carrito add: book1 withQuantity: 1.
	aCartWithBooks add: book2 withQuantity: 1.
	
	self assert: (aCartWithBooks hasItem: book1 withQuantity: 1).
	self assert: (aCartWithBooks hasItem: book2 withQuantity: 1).	
	self assert: (aCartWithBooks size = 2)
! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoEzequielNicols 6/1/2013 16:44'!
testCheckoutOfCartWithSingleElementShouldReturnSinglePrice

	|cart aCashier expectedTotal |
	
	aCashier:= Cashier new.
	cart := self carritoConUnLibro.
	expectedTotal := 2.
	
	self assert: ((aCashier checkOut: cart) total = expectedTotal) ! !

!TusLibrosTest methodsFor: 'cashier' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:48'!
testCheckoutOfEmptyCartShouldRaiseAnError

	|emptyCart aCashier|

	emptyCart := Cart newCartWithCatalog: Bag new.
      aCashier := Cashier new. 

 	self should: [aCashier checkOut: emptyCart ] 
		raise: Error 
		withExceptionDo: [ :anError |  self assert: anError messageText = Cashier cantCheckoutEmptyCartMessage]  ! !

Object subclass: #Cart
	instanceVariableNames: 'id items catalogo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'instance-creation' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:13'!
newCartWithCatalog: aCatalog
	^ self new emptyCartWithCatalog: aCatalog! !

!Cart methodsFor: 'adding' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:00'!
add: anISBN 
	
	
	items add: anISBN! !

!Cart methodsFor: 'adding' stamp: 'EmilianoMenendezEzequielNicols 5/31/2013 19:03'!
add: anISBN withQuantity: aQuantity 
	
	
	(catalogo includesKey: anISBN ) ifFalse:[^ Error signal: 'El libro no pertenece al catlogo' ].
	items add: anISBN withOccurrences: aQuantity ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:01'!
catalog

	^catalogo! !

!Cart methodsFor: 'Initializing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:58'!
emptyCartWithCatalog: aCatalog
	
	items := Bag new.
	catalogo := aCatalog.! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:54'!
hasItem: anISBN withQuantity: aQuantity

	^(items occurrencesOf: anISBN) = aQuantity  ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/27/2013 21:46'!
isEmpty

	^items isEmpty! !

!Cart methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicols 6/1/2013 15:22'!
itemPrice: anItem 
	
	^catalogo at: anItem! !

!Cart methodsFor: 'accessing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:07'!
itemQuantity: anItem  
	
	^items occurrencesOf: anItem! !

!Cart methodsFor: 'testing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:25'!
items

	^items ! !

!Cart methodsFor: 'testing' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 19:32'!
size

	^items size! !

Object subclass: #Cashier
	instanceVariableNames: 'libroDeVentas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 5/29/2013 20:42'!
cantCheckoutEmptyCartMessage
	^ 'Cant checkout an empty cart'! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/1/2013 16:42'!
calculateTotal: aCart 
	
	|cartItems temporaryTotal|
	
	cartItems := aCart items.
	temporaryTotal := 0.
	
	cartItems do: [ :anItem | temporaryTotal := temporaryTotal + (( aCart itemQuantity: anItem) * (aCart itemPrice: anItem))].
	
	^Venta newFrom: aCart withTotal: temporaryTotal.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/1/2013 16:21'!
checkOut: aCart 
	
	(aCart isEmpty) ifTrue: [^ Error signal: self class cantCheckoutEmptyCartMessage].
	^self calculateTotal: aCart.! !

Object subclass: #CreditCard
	instanceVariableNames: 'number owner expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:37'!
newWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate
	^ self new initializeWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate.! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:39'!
expirationDate
	^ expirationDate! !

!CreditCard methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:45'!
initializeWithNumber: aNumber andOwner: anOwner andExpirationDate: anExpirationDate

	(aNumber trim size = 0) ifTrue: [ ^ Error signal: 'Credit card number cannot be empty.' ].
	(anOwner trim size = 0) ifTrue: [ ^ Error signal: 'Credit card owner cannot be empty.' ].
	(anOwner trim size > 30) ifTrue: [ ^ Error signal: 'Credit card owner cannot be greater than 30 characters long.' ].
	
	number := aNumber trim.
	owner := anOwner.
	expirationDate := anExpirationDate.! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:38'!
number
	^ number! !

!CreditCard methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 17:39'!
owner
	^ owner! !

Object subclass: #CreditCardNotExpiredRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardNotExpiredRule methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:12'!
validate: aTransaction
	(aTransaction creditCard expirationDate < Date today)
		ifTrue: [ ^ Error signal: 'EXPIRED_CARD' ].

! !

Object subclass: #CreditCardTransaction
	instanceVariableNames: 'amount creditCard'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardTransaction class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:26'!
newWithAmount: anAmount andCreditCard: aCreditCard 
	^ self new initializeWithAmount: anAmount andCreditCard: aCreditCard.! !

!CreditCardTransaction methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:29'!
amount
	^ amount! !

!CreditCardTransaction methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:30'!
creditCard
	^ creditCard! !

!CreditCardTransaction methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 18:28'!
initializeWithAmount: anAmount andCreditCard: aCreditCard 
	amount := anAmount.
	creditCard := aCreditCard.! !

Object subclass: #CreditCardTransactionResult
	instanceVariableNames: 'code message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCardTransactionResult class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 19:51'!
newWithCode: aCode andMessage: aMessage 
	^ self new initializeWithCode: aCode andMessage: aMessage! !

!CreditCardTransactionResult class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:11'!
newWithError: anError
	^ self newWithCode: '1' andMessage: anError messageText! !

!CreditCardTransactionResult class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 20:22'!
ok
	^ self newWithCode: '0' andMessage: 'OK'! !

!CreditCardTransactionResult methodsFor: 'comparing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:25'!
= anObject

	^ self code = anObject code and: self message = anObject message.
		! !

!CreditCardTransactionResult methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:26'!
code
	^ code! !

!CreditCardTransactionResult methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 20:34'!
initializeWithCode: aCode andMessage: aMessage
	code := aCode.
	message := aMessage.! !

!CreditCardTransactionResult methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:26'!
message
	^ message! !

Object subclass: #MerchantProcessorStub
	instanceVariableNames: 'validationRules'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

MerchantProcessorStub class
	instanceVariableNames: 'validationRules'!

MerchantProcessorStub class
	instanceVariableNames: 'validationRules'!

!MerchantProcessorStub methodsFor: 'operation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:14'!
debit: anAmount fromCard: aCreditCard
	
	|transaction|
	
	transaction := CreditCardTransaction newWithAmount: anAmount andCreditCard: aCreditCard.
	
	[validationRules do: [ :rule | rule validate: transaction]]
		on: Error
		do: [ :error | ^ CreditCardTransactionResult newWithError: error].
						
	^ CreditCardTransactionResult ok.! !

!MerchantProcessorStub methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 19:33'!
initialize
	validationRules := OrderedCollection new.
	validationRules add: PositiveAmountRule new.
	validationRules add: CreditCardNotExpiredRule new.! !

Object subclass: #PositiveAmountRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PositiveAmountRule methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicolas 6/1/2013 21:13'!
validate: aTransaction
	(aTransaction amount <= 0)
		ifTrue: [ ^ Error signal: 'AMOUNT_MUST_BE_GREATER_THAN_ZERO' ].
! !

Object subclass: #Venta
	instanceVariableNames: 'items total'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Venta class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielNicols 6/1/2013 14:43'!
newFrom: aCart

	^ Venta new initializeWith: aCart! !

!Venta class methodsFor: 'as yet unclassified' stamp: 'EmilianoEzequielNicols 6/1/2013 16:42'!
newFrom: aCart withTotal: anAmount

	^ Venta new initializeWith: aCart andTotal: anAmount ! !

!Venta methodsFor: 'initializing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:19'!
initializeWith: aCart 
	
	|cartItems temporaryTotal catalogPrices|
	
	cartItems := aCart items.
	items := Array new: (cartItems size).
	temporaryTotal := 0.
	
	cartItems do: [:anItem | temporaryTotal  := temporaryTotal + ((aCart itemQuantity: anItem) * (aCart itemPrice: anItem))].
	
	total:= temporaryTotal
	
	! !

!Venta methodsFor: 'initializing' stamp: 'EmilianoEzequielNicols 6/1/2013 16:43'!
initializeWith: aCart andTotal: anAmount
	
	|cartItems|
	
	cartItems := aCart items.
	items := Array new: (cartItems size).
	total := anAmount.
	! !

!Venta methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielNicols 6/1/2013 14:36'!
total
	^ total! !
